/*
 * The MIT License
 *
 * Copyright 2017 Germán Bordel <german.bordel at ehu.es>.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.upvehu.gb.ejemplos.scraper;

import edu.upvehu.gb.util.SimpleScraper;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Prueba de SimpleScraper realizada con la Bolsa de Madrid en la web del diario Expansión.
 * 
 * @author Germán Bordel &lt;german.bordel at ehu.es&gt;
 */
public class BolsaScraper {
    
    //Anulamos el constructor
    private BolsaScraper(){}
    
    //En esta prueba ignoraremos ciertos posibles errores dejando una advertencia en un fichero.
    //(al terminar la ejecución queda el fichero en el directorio del proyecto)
    private static final Logger LOGGER = Logger.getLogger("MyLog");  
    static {
        try {  
            LOGGER.addHandler(new FileHandler("MyLogFile.log"));
            LOGGER.setUseParentHandlers(false); //Anulación del "handler" por defecto (volcado a consola)
        } catch (SecurityException | IOException e) {  
            e.printStackTrace();  
        }  
    }
    
    /**
     * Clase auxiliar para almacenar cada uno de los datos obtenidos: instante y valores último-máximo-mínimo.
     */
    private static class Muestra{
        long time;
        double ultimo, maximo, minimmo;
        NumberFormat format = NumberFormat.getInstance(new Locale("es_ES")); //La web formatea separa los decimales con comas

        public Muestra(long time, String ultimo, String maximo, String minimmo) throws ParseException {
            this.time = time;
            this.ultimo = format.parse(ultimo).doubleValue();
            this.maximo = format.parse(maximo).doubleValue();
            this.minimmo = format.parse(minimmo).doubleValue();
        }

        @Override
        public String toString() {
            return "Muestra{"+new Date(time) + ", último=" + ultimo + ", máximo=" + maximo + ", mínimmo=" + minimmo + '}';
        }
    
    }
    
    //El SimpleScraper específico con el que se hará la prueba.
    private static SimpleScraper SCRAPER;
    //Estructura para almacenar los datos: mapa empresa->lista_de_muestras
    private static final Map<String, List<Muestra>> DATOS=new TreeMap<>();
    
    private static void recogerDatos() throws IOException{
        //Se "araña" la web y se toma el instante de tiempo.
        SCRAPER.scrap();
        long time=System.currentTimeMillis();

        //Se recorren los reultados almacenándolos en la estructura al efecto.
        while(SCRAPER.hasNext()) {
            String[] s=SCRAPER.next();
            List<Muestra> l=DATOS.get(s[1]);
            if (l==null) DATOS.put(s[1], l=new ArrayList<>());
            try {
                l.add(new Muestra(time,s[2],s[3],s[4]));
            } catch (ParseException ex) {
                //Si no se puede procesar una muestra, simplemente la ignoramos dejando una entrada de log.
                LOGGER.log(Level.WARNING, s[1]+": "+new Date(time), ex);
            }
        }
    }
    
    /**
     * Prueba de SimpleScraper realizada con la Bolsa de Madrid en la web del diario Expansión.
     * 
     * @param args sin argumentos.
     * @throws java.net.MalformedURLException la URL no es correcta.
     */
    public static void main(String[] args) throws MalformedURLException, IOException {
        SCRAPER=new SimpleScraper(
            "http://www.expansion.com/mercados/cotizaciones/indices/igbolsamadrid_I.MA.html",
                "<td class=\"primera\">.*?>(.*?)</a>.*?"
              + "<td>(.*?)</td>.*?"
              + "<td>(.*?)</td>.*?"
              + "<td>(.*?)</td>"
        );
        
        //Hace dos muestreos de la página web con unos 5 segundos de espera
        System.out.println("Esperando unos 5 segundos a recoger dos muestras de la web...\n");
        recogerDatos();
        try {
            Thread.sleep(5000);
        } catch (InterruptedException ignore) {}
        recogerDatos();
        
        //Muestra de los datos obtenidos por la salida estándar
        for (String empresa:DATOS.keySet())
            for (Muestra m:DATOS.get(empresa))
                System.out.println(empresa+": "+m);
        
        //Se muestra el fichero de log por si se han producido errores.
        System.out.println("\nVOLCADO DE ERRORES\n==================");
        for (String s: Files.readAllLines(new File("MyLogFile.log").toPath())) System.out.println(s);
    }  

}