/*
 * The MIT License
 *
 * Copyright 2017 Germán Bordel <german.bordel at ehu.es>.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package edu.upvehu.gb.util;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.regex.*;

/**
 * Utilidad para extraer información de una página web mediante una expresion regular.
 * 
 * <p>Esta es una clase muy sencilla para servir como ejemplo sin complicaciones. No funciona con el protocolo "https"
 * ni frente a servidores que rechazan peticiones de clientes "no reconocidos" (los navegadores al uso). Todo puede
 * hacerse, pero complicaría el código, lo que es contrario al interés de este ejemplo.</p>
 * 
 * <p>Los objetos se construyen ligados a una página web concreta y para una expresión regular dada,
 * de modo que en cualquier momento puede ejecutarse la obtención de la página, para después ir
 * extrayendo el material requerido conforme al patrón <code>Iterator</code>. Los resutados obtenidos
 * en cada "Match" se proporcionan en un <code>String[]</code> que recoge cada uno de los grupos de 
 * captura de la expresión regular (el 0 es el match completo).</p>
 * 
 * <p>La información de pasos de linea en la página web leida es codificada como "\n" aunque tenga otra codificación en
 * origen. En el procesaiento de la expresión regular el "." incluye los pasos de linea (ver Pattern.DOTALL), y los
 * pasos de linea determinan "matches" para "^"y "$" (ver Pattern.MULTILINE)</p>
 * 
 * @author Germán Bordel &lt;german.bordel at ehu.es&gt;
 */
public class SimpleScraper  implements Iterator<String[]>{
    private final URL url;
    private final Pattern pattern;
    private Matcher matcher;

    /**
     * Genera un nuevo Scraper asociado a una página web concreta y un procesamiento dado.
     * @param url La URL De la página web.
     * @param regEx La expresión regular a aplicar.
     * @throws java.net.MalformedURLException la URL es incorrecta.
     */
    public SimpleScraper(String url, String regEx) throws MalformedURLException {
        this.url = new URL(url);
        this.pattern = Pattern.compile(regEx,Pattern.DOTALL+Pattern.MULTILINE);
    }
    
    /**
     * Devuelve <code>true</code> si la iteración tiene más elementos.
     * @return <code>true</code> si la iteración tiene más elementos.
     */
    @Override public boolean hasNext() {return matcher.find();}

    /**
     * Devuelve el siguiente elemento en la iteración.
     * @return el siguiente elemento en la iteración.
     */
    @Override
    public String[] next() {
        String[] groups=new String[matcher.groupCount()+1];
        for (int i=0;i<groups.length;i++) groups[i]=matcher.group(i);
        return groups;
    }
    
    /**
     * Ejecuta la lectura de la página web de modo que queda en estado preaprado para recoger los 
     * resultados mediante <code>hasNext()-next()</code>
     * @throws IOException problema en la obtención de la página web.
     */
    public void scrap() throws  IOException{
        StringBuffer sb=new StringBuffer();
        try (BufferedReader br = new BufferedReader(new InputStreamReader(url.openStream() ))) {
            String linea;
            while ((linea=br.readLine())!=null) sb.append(linea).append('\n');
        }
        matcher=pattern.matcher(sb);
    }
}